/****************************************************************************
** $Id: cursor.cpp,v 2.3 1998/06/16 11:39:32 warwick Exp $
**
** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
**
** This file is part of an example program for Qt.  This example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/

#include <qwindow.h>
#include <qlabel.h>
#include <qbitmap.h>
#include <qapplication.h>


// cb_bits and cm_bits were generated by X bitmap program.

#define cb_width  32
#define cb_height 32

static unsigned char cb_bits[] = {		// cursor bitmap
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x0f, 0x00,
   0x00, 0x06, 0x30, 0x00, 0x80, 0x01, 0xc0, 0x00, 0x40, 0x00, 0x00, 0x01,
   0x20, 0x00, 0x00, 0x02, 0x10, 0x00, 0x00, 0x04, 0x08, 0x3e, 0x3e, 0x08,
   0x08, 0x03, 0xe0, 0x08, 0xc4, 0x00, 0x00, 0x11, 0x04, 0x1e, 0x78, 0x10,
   0x02, 0x0c, 0x30, 0x20, 0x02, 0x40, 0x00, 0x20, 0x02, 0x40, 0x00, 0x20,
   0x02, 0x40, 0x00, 0x20, 0x02, 0x20, 0x04, 0x20, 0x02, 0x20, 0x04, 0x20,
   0x02, 0x10, 0x08, 0x20, 0x02, 0x08, 0x08, 0x20, 0x02, 0xf0, 0x07, 0x20,
   0x04, 0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x10, 0x08, 0x00, 0xc0, 0x08,
   0x08, 0x3c, 0x30, 0x08, 0x10, 0xe6, 0x19, 0x04, 0x20, 0x00, 0x0f, 0x02,
   0x40, 0x00, 0x00, 0x01, 0x80, 0x01, 0xc0, 0x00, 0x00, 0x06, 0x30, 0x00,
   0x00, 0xf8, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00};

#define cm_width  32
#define cm_height 32

static unsigned char cm_bits[] = {		// cursor bitmap mask
   0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x1f, 0x00, 0x00, 0xfe, 0x3f, 0x00,
   0x80, 0x07, 0xf0, 0x00, 0xc0, 0x01, 0xc0, 0x01, 0x60, 0x00, 0x00, 0x03,
   0x30, 0x00, 0x00, 0x06, 0x18, 0x00, 0x00, 0x0c, 0x0c, 0x3e, 0x3e, 0x18,
   0x0e, 0x03, 0xe0, 0x18, 0xc6, 0x00, 0x00, 0x31, 0x07, 0x1e, 0x78, 0x30,
   0x03, 0x0c, 0x30, 0x60, 0x03, 0x40, 0x00, 0x60, 0x03, 0x40, 0x00, 0x60,
   0x03, 0x40, 0x00, 0x60, 0x03, 0x20, 0x04, 0x60, 0x03, 0x20, 0x04, 0x60,
   0x03, 0x10, 0x08, 0x60, 0x03, 0x08, 0x08, 0x60, 0x03, 0xf0, 0x07, 0x60,
   0x06, 0x00, 0x00, 0x30, 0x06, 0x00, 0x00, 0x30, 0x0c, 0x00, 0xc0, 0x18,
   0x0c, 0x3c, 0x30, 0x18, 0x18, 0xe6, 0x19, 0x0c, 0x30, 0x00, 0x0f, 0x06,
   0x60, 0x00, 0x00, 0x03, 0xc0, 0x01, 0xc0, 0x01, 0x80, 0x07, 0xf0, 0x00,
   0x00, 0xfe, 0x3f, 0x00, 0x00, 0xf8, 0x0f, 0x00};


//
// The CursorView contains many labels with different cursors.
//

class CursorView : public QWindow		// cursor view
{
public:
    CursorView();
};

//
// Constructs a cursor view.
//

CursorView::CursorView()			// construct view
{
    static struct {
	QCursor *cursor;			// cursor
	char 	*name;				// cursor name
    } list[] = {
	{ (QCursor *)&arrowCursor,	"arrowCursor" },
	{ (QCursor *)&upArrowCursor,	"upArrowCursor" },
	{ (QCursor *)&crossCursor,	"crossCursor" },
	{ (QCursor *)&waitCursor,	"waitCursor" },
	{ (QCursor *)&ibeamCursor,	"ibeamCursor" },
	{ (QCursor *)&sizeVerCursor,	"sizeVerCursor" },
	{ (QCursor *)&sizeHorCursor,	"sizeHorCursor" },
	{ (QCursor *)&sizeBDiagCursor,	"sizeBDiagCursor" },
	{ (QCursor *)&sizeFDiagCursor,	"sizeFDiagCursor" },
	{ (QCursor *)&sizeAllCursor,	"sizeAllCursor" },
	{ 0,				0 } };

    setCaption( "CursorView" );			// set window caption

    QLabel *label;
    int i=0;
    for ( int y=0; y<2; y++ ) {			// create 10 small labels
	for ( int x=0; x<5; x++ ) {
	    label = new QLabel( this );
	    label->setCursor( *list[i].cursor );
	    label->setText( list[i].name );
	    label->setAlignment( AlignCenter );
	    label->setFrameStyle( QFrame::Box | QFrame::Raised );
	    label->setGeometry( 10+x*110, 10+y*50, 100, 40 );
	    i++;
	}
    }

    QBitmap cb( cb_width, cb_height, cb_bits, TRUE );
    QBitmap cm( cm_width, cm_height, cm_bits, TRUE );
    QCursor custom( cb, cm );			// create bitmap cursor

    label = new QLabel( this );			// create the big label
    label->setCursor( custom );
    label->setText( "Custom bitmap cursor" );
    label->setAlignment( AlignCenter );
    label->setFrameStyle( QFrame::Box | QFrame::Sunken );
    label->setGeometry( 10, 110, 540, 40 );

    resize( 20+5*110, 20+3*50 );
}


//
// Create and display a CursorView.
//

int main( int argc, char **argv )
{
    QApplication a( argc, argv );		// application object
    CursorView   v;				// cursor view
    a.setMainWidget( &v );
    v.show();
    return a.exec();
}
